//
#ifndef MIB_H
#define MIB_H

typedef unsigned long oid;

typedef std::pair<int, std::string> EnumInts;
typedef std::pair<int, std::string> Subid;
typedef std::list<EnumInts> EnumList;

/*
 * A linked list of nodes.
 */
class CMIBNode {
public:
	CMIBNode();
	CMIBNode(const CMIBNode& r);
	static void CreateTree(std::list<CMIBNode>&, LPSTR&);
	int GetHash();
	inline bool IsParent(std::string& s) { return s == m_parent; }
	inline bool IsLabel(std::string& s) { return s == m_label; }
	inline std::string& GetLabel() { return m_label; }
	inline int GetSubid() const { return subid; }
	inline EnumList& GetEnums() { return enums; }
	inline void ClearEnums() { enums.clear(); }
	inline std::string& GetDescription() { return description; }
	inline std::string& GetParent() { return m_parent; }
	inline void ClearDescription() { description.erase(); }
	inline int GetType() const { return type; }
protected:
	static bool parse_mib_header(LPSTR&, std::string&);
	static bool parse_asntype(LPSTR& fp, std::string& name, int& ntype, std::string& ntoken);
	static void parse_objectid(std::list<CMIBNode>&, LPSTR& fp, std::string& name);
	static void parse_objectgroup(std::list<CMIBNode>&, LPSTR& fp, std::string& name);
	static void parse_objecttype(std::list<CMIBNode>& list, LPSTR& fp, std::string& token);
	static void parse_compliance(std::list<CMIBNode>& list, LPSTR& fp, std::string& name);
	static void parse_moduleIdentity(std::list<CMIBNode>& list, LPSTR& fp, std::string& name);
	static void parse_notificationDefinition(std::list<CMIBNode>& list, LPSTR& fp, std::string& name);
	static int getoid(LPSTR& fp, Subid* oid,  int length);
private:
	std::string m_label;  /* This node's (unique) textual name */
    long  subid;  /* This node's integer subidentifier */
	std::string m_parent; /* The parent's textual name */
    int type;	    /* The type of object this represents */
	EnumList enums; 	/* (optional) list of enumerated (otherwise NULL) */
	std::string description;	/* description (a quoted string) */
};

typedef std::list<CMIBNode> MIBList;
typedef std::list<CMIBNode>::iterator MIBListIter;
typedef std::list<CMIBNode>::reverse_iterator MIBListRIter;


class CMIB {
public:
	CMIB();
	CMIB(const char* label, int subid = 0, CMIB* pParent = NULL);
	~CMIB();
	static CMIB* CreateTree(LPSTR&);
	static void DestroyTree(CMIB*);
	void ParseSubTree(LPSTR&, std::vector<oid>&);
	static CMIB* GetSymbol(oid*, int, CMIB*, std::string&);
	enum
	{
		NHASHSIZE = 128,
		MAX_SUBID = 0xFFFFFFFF,
	};
	inline static int NBUCKET(int x)  { return (x & 0x7F); }

#ifdef _DEBUG
	void print_subtree(int count);
	static void print_node_hash();
#endif
private:
	static int lc_cmp(const char*, const char*);
	//
	static int translation_table[];
	static bool s_tableInitialized;
	static MIBList nbuckets[];
	static void init_node_hash(MIBList&);
	static void build_translation_table();
	void do_subtree();

	CMIB* child_list;
	CMIB* parent;
	CMIB* next_peer;
	std::string label;			/* This node's textual name */
    oid subid;		/* This node's integer subidentifier */
    int type;			/* This node's object type */
	EnumList enums;	/* (optional) list of enumerated integers (otherwise NULL) */
	std::string description;	/* description (a quoted string) */
};

typedef std::list<CMIB> MIBTree;
typedef std::list<CMIB>::iterator MIBTreeIter;

#endif

